@@ -2,6 +2,8 @@ import numpy as np
 from sklearn.metrics import accuracy_score
 from operator import itemgetter
 from scipy.spatial import distance
+import matplotlib.pyplot as plt
+from scipy.stats import norm, gaussian_kde
 
 
 def accuracy_rate1(x, y):
@@ -25,6 +27,22 @@ def rejection_rate(x, y):
     return z
 
 
+def get_array(x):
+    return [np.array(i) for i in x]
+
+
+def get_pdf(x, y, z):
+    return 1 / (x * np.sqrt(2 * np.pi)) * np.exp(-(y - z) ** 2 / (2 * x ** 2))
+
+
+def get_iqr(x, y):
+    return 1.5 * (x - y)
+
+
+def get_percentile(x, y):
+    return np.percentile(x, y)
+
+
 class LabelSecurity:
     """
     Label Security
@@ -755,5 +773,216 @@ class ProtoCertt:
         return empty
 
 
+class visualize:
+    """
+    confidence_list : list
+    2D array: Contains list of lists for all label securities
+
+    num_classes : int :
+    The cardinality of the class labels
+
+    color: list :
+    contains colors and cardinality of this list be same as the num_classes
+
+    class_labels : list
+    list of class labels
+
+    :return a visualization of the label securities
+    """
+
+    def __init__(self, confidence_list, num_classes, colors, class_labels, n_bins=100):
+        self.confidence = confidence_list
+        self.num_classes = num_classes
+        self.colors = colors
+        self.labels = class_labels
+        self.n_bins = n_bins
+
+        if type(self.confidence) != list:
+            raise TypeError('Improper data type passed for confidence_list.')
+
+        if num_classes < 1:
+            raise ValueError('Number of classes must be atleast 1.')
+
+        if type(self.labels) != list:
+            raise TypeError('Improper data type passed for class labels.')
+
+        if len(colors) != num_classes:
+            raise ValueError(f'The must be {len(num_classes)} different colors')
+
+    def get_vis(self, x):
+        """
+        :param x: str: title of the plot
+        :return: visualization of the classification label securities
+        """
+
+        # Creating dataset for box plot
+        data = get_array(x=self.confidence)
+
+        fig = plt.figure(figsize=(10, 7))
+        ax = fig.add_subplot(111)
+
+        # Creating axes instance
+        bp = ax.boxplot(data, patch_artist=True, notch='True', vert=0)
+        colors = self.colors
+        for patch, color in zip(bp['boxes'], colors):
+            patch.set_facecolor(color)
+
+        # changing color and linewidth of whiskers
+        for whisker in bp['whiskers']:
+            whisker.set(color='#8B008B', linewidth=1.5, linestyle=":")
+
+        # changing color and linewidth of caps
+        for cap in bp['caps']:
+            cap.set(color='#8B008B', linewidth=2)
+
+        # changing color and linewidth of medians
+        for median in bp['medians']:
+            median.set(color='red', linewidth=3)
+
+        # changing style of fliers
+        for flier in bp['fliers']:
+            flier.set(marker='D', color='#e7298a', alpha=0.5)
+
+        # x-axis labels
+        ax.set_yticklabels(self.labels)
+
+        # Adding title
+        plt.title(x)
+
+        # Removing top axes and right axes ticks
+        ax.get_xaxis().tick_bottom()
+        ax.get_yaxis().tick_left()
+
+        # show plot
+        return plt.show()
+
+    def get_vis_(self, x, y, z):
+        """
+        :param x: title of the x-axis
+        :param y: title of y-axis
+        :param z: title of the plot
+        :return: visualize the distribution of the classification label securities of all the classes
+        """
+
+        for i in range(len(self.confidence)):
+            mu = np.mean(self.confidence[i])
+            sigma = np.std(self.confidence[i])
+            density = gaussian_kde(self.confidence[i])
+            fig, axes = plt.subplots(nrows=2, ncols=1, sharex='all', figsize=(10, 7))
+
+            # histogram
+            axes[1].hist(self.confidence[i], self.n_bins, density=True, alpha=.1, edgecolor='black')
+
+            median = np.percentile(self.confidence[i], 50)
+            q1 = np.percentile(self.confidence[i], 25)
+            q3 = np.percentile(self.confidence[i], 75)
+
+            # probability density function
+            x_ = np.linspace(np.percentile(self.confidence[i], 0), np.percentile(self.confidence[i], 100),
+                             self.n_bins)
+
+            density.covariance_factor = lambda: 100
+
+            # KDE : non-parametric estimation of PDF of random variable
+            axes[1].plot(x_, density(x_))
+
+            # add text to bottom graph.
+            axes[1].annotate('q1', xy=(q1, norm(mu, sigma).pdf(q1)), ha='center')
+            axes[1].annotate('q3', xy=(q3, norm(mu, sigma).pdf(q3)), ha='center')
+
+            # dashed lines
+            plt.axvline(np.percentile(self.confidence[i], 0), color='b', linestyle='-.', alpha=.6, linewidth=.5)
+            plt.axvline(np.percentile(self.confidence[i], 25), color='g', linestyle='--', alpha=.6, linewidth=.5)
+            plt.axvline(np.percentile(self.confidence[i], 50), color='orange', linestyle='--', alpha=.6,
+                        linewidth=.5)
+            plt.axvline(np.percentile(self.confidence[i], 75), color='b', linestyle='--', alpha=.6, linewidth=.5)
+            plt.axvline(np.percentile(self.confidence[i], 100), color='r', linestyle='-.', alpha=.6, linewidth=.5)
+
+            axes[1].set_ylabel(y)
+            axes[1].set_xlabel(x)
+
+            # top boxplot
+            axes[0].set_title(f'{z} {self.labels[i]}')
+            axes[0].boxplot(self.confidence[i], 0, 'gD', vert=False)
+            axes[0].axvline(median, color='orange', alpha=.6, linewidth=.5)
+            axes[0].axvline(q1, color='green', alpha=.6, linewidth=.5)
+            axes[0].axvline(q3, color='b', alpha=.6, linewidth=.5)
+            axes[0].axvline(np.percentile(self.confidence[i], 0), color='b', alpha=.6, linewidth=.5)
+            axes[0].axvline(np.percentile(self.confidence[i], 100), color='r', alpha=.6, linewidth=.5)
+            axes[0].axis('off')
+
+            plt.subplots_adjust(hspace=0)
+        return plt.show()
+
+    def get_vis__(self, x, y, z):
+        """
+        :param x: str: label of the x-axis
+        :param y: str: label of the y-axis
+        :param z: str: title of plot
+        :return: visualization the distribution(pdf) for the classification labels securities of all classes
+
+        """
+
+        for i in range(len(self.confidence)):
+            mu = np.mean(self.confidence[i])
+            sigma = np.std(self.confidence[i])
+
+            fig, axes = plt.subplots(nrows=2, ncols=1, sharex='all', figsize=(10, 7))
+
+            # histogram
+            n, bins, patches = axes[1].hist(self.confidence[i], self.n_bins, density=True, alpha=.1,
+                                            edgecolor='black')
+
+            pdf = get_pdf(x=sigma, y=bins, z=mu)
+
+            median = np.percentile(self.confidence[i], 50)
+            q1 = np.percentile(self.confidence[i], 25)
+            q3 = np.percentile(self.confidence[i], 75)
+
+            # probability density function
+            axes[1].plot(bins, pdf, color='orange', alpha=.6)
+            iqr = get_iqr(x=q3, y=q1)
+            x1 = np.linspace(q1 - iqr, q1)
+            x2 = np.linspace(q3, q3 + iqr)
+            pdf1 = get_pdf(x=sigma, y=x1, z=mu)
+            pdf2 = get_pdf(x=sigma, y=x2, z=mu)
+
+            axes[1].fill_between(x1, pdf1, 0, alpha=.6, color='orange')
+            axes[1].fill_between(x2, pdf2, 0, alpha=.6, color='orange')
+
+            # add text to bottom graph.
+            axes[1].annotate("{:.1f}%".format(100 * (norm(mu, sigma).cdf(q1) - norm(mu, sigma).cdf(q1 - iqr))),
+                             xy=(q1 - iqr / 2, 0), ha='center')
+            axes[1].annotate("{:.1f}%".format(100 * (norm(mu, sigma).cdf(q3) - norm(mu, sigma).cdf(q1))),
+                             xy=(median, 0), ha='center')
+            axes[1].annotate("{:.1f}%".format(100 * (norm(mu, sigma).cdf(q3 + iqr) - norm(mu, sigma).cdf(q3))),
+                             xy=(q3 + iqr / 2, 0), ha='center')
+            axes[1].annotate('q1', xy=(q1, norm(mu, sigma).pdf(q1)), ha='center')
+            axes[1].annotate('q3', xy=(q3, norm(mu, sigma).pdf(q3)), ha='center')
+
+            # dashed lines
+            plt.axvline(np.percentile(self.confidence[i], 0), color='b', linestyle='-.', alpha=.6, linewidth=.5)
+            plt.axvline(np.percentile(self.confidence[i], 25), color='g', linestyle='--', alpha=.6, linewidth=.5)
+            plt.axvline(np.percentile(self.confidence[i], 50), color='orange', linestyle='--', alpha=.6, linewidth=.5)
+            plt.axvline(np.percentile(self.confidence[i], 75), color='b', linestyle='--', alpha=.6, linewidth=.5)
+            plt.axvline(np.percentile(self.confidence[i], 100), color='r', linestyle='-.', alpha=.6, linewidth=.5)
+
+            axes[1].set_ylabel(y)
+            axes[1].set_xlabel(x)
+
+            # top boxplot
+            axes[0].set_title(f'{z} {self.labels[i]}')
+            axes[0].boxplot(self.confidence[i], 0, 'gD', vert=False)
+            axes[0].axvline(median, color='orange', alpha=.6, linewidth=.5)
+            axes[0].axvline(q1, color='green', alpha=.6, linewidth=.5)
+            axes[0].axvline(q3, color='b', alpha=.6, linewidth=.5)
+            axes[0].axvline(np.percentile(self.confidence[i], 0), color='b', alpha=.6, linewidth=.5)
+            axes[0].axvline(np.percentile(self.confidence[i], 100), color='r', alpha=.6, linewidth=.5)
+            axes[0].axis('off')
+
+            plt.subplots_adjust(hspace=0)
+        return plt.show()
+
+
 if __name__ == '__main__':
     print('import module to use')
